#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Development Extraction Running Script 
#------------------------------------------------------------------------------
# Script to run the extraction process (development)
#==============================================================================

import subprocess, os
from datetime import datetime

INPUT_DIR_PATH = f'{os.getcwd()}/input/'
OUTPUT_DIR_PATH = f'{os.getcwd()}/output/'

from context import tenet


#==============================================================================
# Data
#==============================================================================

# Input Samples
samples_dir = "samples/"
srsa_ip_sample = samples_dir + "SRSA-IP/"
petit_prince_sample = samples_dir + "le-petit-prince/"
petit_prince_sample_0 = petit_prince_sample + "chapitre-0/"
petit_prince_sample_1 = petit_prince_sample + "chapitre-1/"

# Input Dev (UNL)
corpus_40 = "dev/CCTP-SRSA-IP-20210831/"
req_100 = "dev/CCTP-SRSA-IP-20210831-R100/"
req_200 = "dev/CCTP-SRSA-IP-20210831-R200/"
req_300 = "dev/CCTP-SRSA-IP-20210831-R300/"
req_1100 = "dev/CCTP-SRSA-IP-20210831-R1100/"
req_13900 = "dev/CCTP-SRSA-IP-20210831-R13900/"
corpus_ERTMS = "dev/ERTMS/"
corpus_PEV = "dev/PEV-RSE-Approach/"
eolss_fish = "dev/EOLSS-FISH/"

   
#==============================================================================
# AMR Extraction
#==============================================================================

# Input Data
test_data_dir = f'{INPUT_DIR_PATH}amrDocuments/'
uuid_num = '2'
amrld_dir_path = f'{test_data_dir}dev/solar-system-{uuid_num}/'
amrld_file_path = f'{amrld_dir_path}SSC-02-01.stog.amr.ttl'

# amrld_dir_path = f'{test_data_dir}samples/solar-system/'
# onto_prefix = 'SolarSystemSample'

# Output references
onto_prefix = f'SolarSystemDev{uuid_num}'
time_ref = f'{datetime.now().strftime("%Y%m%d")}'
out_dir_path = f'{OUTPUT_DIR_PATH}{onto_prefix}-{time_ref}/'
os.makedirs(out_dir_path, exist_ok=True)
out_file_name = f'{onto_prefix}_factoid.ttl'
out_file_path = f'{out_dir_path}{out_file_name}'
technical_dir_path = f'{out_dir_path}technical-data/'

# tenet_process = ["python3", "extract.py", 
#                       "--source_type", 'amr', 
#                       "--source_corpus", amrld_file_path,
#                       "--target_id", onto_prefix ]
# tenet_process += ["--base_output_dir", out_dir_path]
# tenet_process += ["--engine", 'tenet']
# subprocess.run(tenet_process)

factoids = tenet.create_ontology_from_amrld_file(amrld_file_path, 
                                                  onto_prefix=onto_prefix,
                                                  out_file_path=out_file_path,
                                                  technical_dir_path=technical_dir_path)


# factoids = tenet.create_ontology_from_amrld_dir(amrld_dir_path, 
#                                                 onto_prefix=onto_prefix,
#                                                 out_file_path=out_file_path,
#                                                 technical_dir_path=technical_dir_path)

# print(factoids)


#==============================================================================
# UNL Extraction
#==============================================================================

# -- Target: default-ontology
# base_ontology_path = f'{INPUT_DIR_PATH}targetFrameStructure/default-ontology.ttl'
# ontology_seed_path = f'{INPUT_DIR_PATH}targetFrameStructure/default-ontology-seed.ttl'
# #target_ontology_namespace = "https://tenet.tetras-libre.fr/default-ontology/"

# unlrdf_file_path = f'{INPUT_DIR_PATH}/unlDocuments/' + req_100

# tenet.main.create_ontology_from_unlrdf_file(unlrdf_file_path, 
#                                             base_ontology_path=base_ontology_path,
#                                             ontology_seed_path=ontology_seed_path,
#                                             #onto_prefix=target_ontology_namespace,
#                                             out_file_path=OUTPUT_DIR_PATH)